using System;
using System.Linq;
using AutomobileSampleDataModel.Helpers;

namespace AutomobileSampleDataModel
{
    public class AutomobileRepository
    {
        private static AutomobileRepository _repository;
        public static AutomobileRepository Instance
        {
            get
            {
                if (_repository == null)
                {
                    _repository = new AutomobileRepository();
                }
                return _repository;
            }
        }

        private readonly Automobiles _automobiles;

        public AutomobileRepository()
        {
            _automobiles = new Automobiles(SampleData.GetAllAutomobiles());
        }

        public Years GetYears(Func<object, string> linkBuilder)
        {
            return (from a in _automobiles
                    group a by a.Year into y
                    select new Year
                            {
                                Value = y.Key,
                                Uri = linkBuilder(new { year = y.Key })
                            }).ToYears();
        }

        public Makes GetMakes(int year, Func<object, string> linkBuilder)
        {
            return (from a in _automobiles
                    where a.Year == year
                    group a by a.Make into m
                    select new Make
                            {
                                Year = year,
                                Name = m.Key,
                                Uri = linkBuilder(new { year, make = m.Key })
                            }).ToMakes();
        }

        public Models GetModels(int year, string make, Func<object, string> linkBuilder)
        {
            return (from a in _automobiles
                    where a.Year == year
                          && a.Make.Equals(make, StringComparison.OrdinalIgnoreCase)
                    group a by a.Model into m
                    select new Model
                            {
                                Year = year,
                                Make = make,
                                Name = m.Key,
                                Uri = linkBuilder(new { year, make, model = m.Key })
                            }).ToModels();
        }

        public Automobiles GetEngines(int year, string make, string model, Func<object, string> linkBuilder)
        {
            return (from a in _automobiles
                    where a.Year == year
                          && a.Make.Equals(make, StringComparison.OrdinalIgnoreCase)
                          && a.Model.Equals(model, StringComparison.OrdinalIgnoreCase)
                    select new Automobile
                               {
                                   Id = a.Id,
                                   Year = year,
                                   Make = make,
                                   Model = model,
                                   Engine = a.Engine,
                                   Uri = linkBuilder(new { year, make, model, engine = a.Engine })
                               }).ToAutomobiles();
        }

        public Automobile GetAutomobile(int year, string make, string model, string engine, Func<object, string> linkBuilder)
        {
            return (from a in _automobiles
                    where a.Year == year 
                          && a.Make.Equals(make, StringComparison.OrdinalIgnoreCase)
                          && a.Model.Equals(model, StringComparison.OrdinalIgnoreCase)
                          && a.Engine.Equals(engine, StringComparison.OrdinalIgnoreCase)
                    select new Automobile
                               {
                                   Id = a.Id,
                                   Year = year,
                                   Make = make,
                                   Model = model,
                                   Engine = a.Engine,
                                   Uri = linkBuilder(new { year, make, model, engine = a.Engine })
                               }).FirstOrDefault();
        }

        public int Insert(Automobile automobile)
        {
            automobile.Id = GetNextId();
            _automobiles.Add(automobile);

            return automobile.Id;
        }

        public int Update(int year, string make, string model, string engine, Automobile updatedAutomobile)
        {
            var updateItem = (from a in _automobiles
                              where a.Year == year
                                    && a.Make.Equals(make, StringComparison.OrdinalIgnoreCase)
                                    && a.Model.Equals(model, StringComparison.OrdinalIgnoreCase)
                                    && a.Engine.Equals(engine, StringComparison.OrdinalIgnoreCase)
                              select a).FirstOrDefault();

            if (updateItem == null)
            {
                return 0;
            }

            updateItem.Year = updatedAutomobile.Year;
            updateItem.Make = updatedAutomobile.Make;
            updateItem.Model = updatedAutomobile.Model;
            updateItem.Engine = updatedAutomobile.Engine;

            return 1;
        }

        public bool Delete(int year, string make, string model, string engine)
        {
            var result = _automobiles.RemoveAll(a => a.Year == year
                                                     && a.Make.Equals(make, StringComparison.OrdinalIgnoreCase)
                                                     && a.Model.Equals(model, StringComparison.OrdinalIgnoreCase)
                                                     && a.Engine.Equals(engine, StringComparison.OrdinalIgnoreCase));
            if (result == 0)
            {
                return false;
            }
            return true;
        }

        private int GetNextId()
        {
            return (from a in _automobiles
                    select a.Id).Max() + 1;
        }
    }
}