using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Web;
using AutomobileSampleDataModel;
using WcfRestUtilities;

namespace AutomobileService
{
    [ServiceBehavior(ConcurrencyMode=ConcurrencyMode.Single)]
    public class AutomobileService : IAutomobileService
    {
        #region Xml Entry Points
        public Years GetYears()
        {
            return GetYearsInternal();
        }

        public Makes GetMakes(string year)
        {
            return GetMakesInternal(year);
        }

        public Models GetModels(string year, string make)
        {
            return GetModelsInternal(year, make);
        }

        public Automobiles GetAutomobiles(string year, string make, string model)
        {
            return GetAutomobilesInternal(year, make, model);
        }

        public Automobile GetAutomobile(string year, string make, string model, string engine)
        {
            return GetAutomobileInternal(year, make, model, engine);
        }

        public Automobile InsertAutomobile(Automobile automobile)
        {
            return InsertAutomobileInternal(automobile);
        }

        public Automobile UpdateAutomobile(string year, string make, string model, string engine, Automobile automobile)
        {
            return UpdateAutomobileInternal(year, make, model, engine, automobile);
        }

        public void DeleteAutomobile(string year, string make, string model, string engine)
        {
            DeleteAutomobileInternal(year, make, model, engine);
        } 
        #endregion

        #region Json Entry Points
        public Years GetJsonYears()
        {
            return GetYearsInternal();
        }

        public Makes GetJsonMakes(string year)
        {
            return GetMakesInternal(year);
        }

        public Models GetJsonModels(string year, string make)
        {
            return GetModelsInternal(year, make);
        }

        public Automobiles GetJsonAutomobiles(string year, string make, string model)
        {
            return GetAutomobilesInternal(year, make, model);
        }

        public Automobile GetJsonAutomobile(string year, string make, string model, string engine)
        {
            return GetAutomobileInternal(year, make, model, engine);
        }

        public Automobile InsertJsonAutomobile(Automobile automobile)
        {
            return InsertAutomobileInternal(automobile);
        }

        public Automobile UpdateJsonAutomobile(string year, string make, string model, string engine, Automobile automobile)
        {
            return UpdateAutomobileInternal(year, make, model, engine, automobile);
        }

        public void DeleteJsonAutomobile(string year, string make, string model, string engine)
        {
            DeleteAutomobileInternal(year, make, model, engine);
        }
        #endregion

        #region Internal "Logic" functions
        private Years GetYearsInternal()
        {
            return AutomobileRepository.Instance.GetYears(GetYearLink);
        }

        private Makes GetMakesInternal(string year)
        {
            int y = 0;
            if (!int.TryParse(year, out y))
            {
                RestHelpers.SetStatusNotFound();
                return null;
            }

            return AutomobileRepository.Instance.GetMakes(y, GetMakeLink);
        }

        private Models GetModelsInternal(string year, string make)
        {
            int y = 0;
            if (!int.TryParse(year, out y))
            {
                RestHelpers.SetStatusNotFound();
                return null;
            }

            return AutomobileRepository.Instance.GetModels(y, make, GetModelLink);
        }

        private Automobiles GetAutomobilesInternal(string year, string make, string model)
        {
            int y = 0;
            if (!int.TryParse(year, out y))
            {
                RestHelpers.SetStatusNotFound();
                return null;
            }

            return AutomobileRepository.Instance.GetEngines(y, make, model, GetEngineLink);
        }

        private Automobile GetAutomobileInternal(string year, string make, string model, string engine)
        {
            int y = 0;
            if (!int.TryParse(year, out y))
            {
                RestHelpers.SetStatusNotFound();
                return null;
            }

            var result = AutomobileRepository.Instance.GetAutomobile(y, make, model, engine, GetEngineLink);
            if (result == null)
            {
                RestHelpers.SetStatusNotFound();
                return null;
            }
            return result;
        }

        private Automobile InsertAutomobileInternal(Automobile automobile)
        {
            if (automobile == null)
            {
                RestHelpers.SetStatusBadRequest();
                return null;
            }

            var result = AutomobileRepository.Instance.Insert(automobile);
            if (result == 0)
            {
                RestHelpers.SetStatusNotFound();
                return null;
            }
            else
            {
                RestHelpers.SetStatusCreated();
                return GetAutomobileInternal(automobile.Year.ToString(), automobile.Make, automobile.Model, automobile.Engine);
            }
        }

        private Automobile UpdateAutomobileInternal(string year, string make, string model, string engine, Automobile automobile)
        {
            int y = 0;
            if (!int.TryParse(year, out y))
            {
                RestHelpers.SetStatusNotFound();
                return null;
            }

            if (automobile == null)
            {
                RestHelpers.SetStatusBadRequest();
                return null;
            }

            var result = AutomobileRepository.Instance.Update(y, make, model, engine, automobile);
            if (result == 0)
            {
                RestHelpers.SetStatusNotFound();
                return null;
            }
            else
            {
                return GetAutomobile(automobile.Year.ToString(), automobile.Make, automobile.Model, automobile.Engine);
            }
        }

        private void DeleteAutomobileInternal(string year, string make, string model, string engine)
        {
            int y = 0;
            if (!int.TryParse(year, out y))
            {
                RestHelpers.SetStatusNotFound();
                return;
            }

            var result = AutomobileRepository.Instance.Delete(y, make, model, engine);
            if (result)
            {
                RestHelpers.SetStatusOK();
                return;
            }
            else
            {
                RestHelpers.SetStatusNotFound();
                return;
            }
        }
        #endregion

        #region Get Relative Links
        public bool IsJsonRequest
        {
            get
            {
                return WebOperationContext.Current.IncomingRequest.UriTemplateMatch.Template.ToString().Contains("/json");
            }
        }

        public string GetYearLink(object values)
        {
            var template = IsJsonRequest ? AutomobileServiceConstants.MakesJson : AutomobileServiceConstants.Makes;
            return RestHelpers.GetLink(template, values).ToString();
        }

        public string GetMakeLink(object values)
        {
            var template = IsJsonRequest ? AutomobileServiceConstants.ModelsJson : AutomobileServiceConstants.Models;
            return RestHelpers.GetLink(template, values).ToString();
        }

        public string GetModelLink(object values)
        {
            var template = IsJsonRequest ? AutomobileServiceConstants.EnginesJson : AutomobileServiceConstants.Engines;
            return RestHelpers.GetLink(template, values).ToString();
        }

        public string GetEngineLink(object values)
        {
            var template = IsJsonRequest ? AutomobileServiceConstants.EngineJson : AutomobileServiceConstants.Engine;
            return RestHelpers.GetLink(template, values).ToString();
        }
        #endregion

        /// <summary>
        /// This is a demo page which returns a basic html document for use during the demo only
        /// </summary>
        /// <returns>An Html Page with links to sample data.</returns>
        public Message GetDemoPage()
        {
            return RestHelpers.CreateMessageFromFile("demo.html", "text/html");
        }
    }
}