using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Web;
using AutomobileSampleDataModel;

namespace AutomobileService
{
    [ServiceContract]
    public interface IAutomobileService
    {
        #region Xml Contracts
        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.YearsTemplate)]
        [XmlSerializerFormat]
        Years GetYears();

        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.MakesTemplate)]
        [XmlSerializerFormat]
        Makes GetMakes(string year);

        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.ModelsTemplate)]
        [XmlSerializerFormat]
        Models GetModels(string year, string make);

        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.EnginesTemplate)]
        [XmlSerializerFormat]
        Automobiles GetAutomobiles(string year, string make, string model);

        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.EngineTemplate)]
        [XmlSerializerFormat]
        Automobile GetAutomobile(string year, string make, string model, string engine);

        [OperationContract]
        [WebInvoke(UriTemplate = AutomobileServiceConstants.YearsTemplate)]
        [XmlSerializerFormat]
        Automobile InsertAutomobile(Automobile automobile);

        [OperationContract]
        [WebInvoke(Method = "PUT", UriTemplate = AutomobileServiceConstants.EngineTemplate)]
        [XmlSerializerFormat]
        Automobile UpdateAutomobile(string year, string make, string model, string engine, Automobile automobile);

        [OperationContract]
        [WebInvoke(Method = "DELETE", UriTemplate = AutomobileServiceConstants.EngineTemplate)]
        [XmlSerializerFormat]
        void DeleteAutomobile(string year, string make, string model, string engine); 
        #endregion

        #region Json Contracts
        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.YearsJsonTemplate, RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json)]
        Years GetJsonYears();

        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.MakesJsonTemplate, RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json)]
        Makes GetJsonMakes(string year);

        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.ModelsJsonTemplate, RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json)]
        Models GetJsonModels(string year, string make);

        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.EnginesJsonTemplate, RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json)]
        Automobiles GetJsonAutomobiles(string year, string make, string model);

        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.EngineJsonTemplate, RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json)]
        Automobile GetJsonAutomobile(string year, string make, string model, string engine);

        [OperationContract]
        [WebInvoke(UriTemplate = AutomobileServiceConstants.YearsJsonTemplate, RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json)]
        Automobile InsertJsonAutomobile(Automobile automobile);

        [OperationContract]
        [WebInvoke(Method = "PUT", UriTemplate = AutomobileServiceConstants.EngineJsonTemplate, RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json)]
        Automobile UpdateJsonAutomobile(string year, string make, string model, string engine, Automobile automobile);

        [OperationContract]
        [WebInvoke(Method = "DELETE", UriTemplate = AutomobileServiceConstants.EngineJsonTemplate, RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json)]
        void DeleteJsonAutomobile(string year, string make, string model, string engine); 
        #endregion

        /*
         * Demo Page Only, Not part of the Sample
         */
        [OperationContract]
        [WebGet(UriTemplate = AutomobileServiceConstants.DemoTemplate)]
        Message GetDemoPage();
    }
}