using System;
using System.Collections.Generic;
using System.Reflection;

namespace CatchAllAutomobileService
{
    public static class AutomobileServiceConstants
    {
        public const string DemoTemplate = "/demo.html";

        public const string YearsTemplate = "/";
        public const string MakesTemplate = "/{year}";
        public const string ModelsTemplate = "/{year}/{make}";
        public const string EnginesTemplate = "/{year}/{make}/{model}";
        public const string EngineTemplate = "/{year}/{make}/{model}/{engine}";

        public const string YearsJsonTemplate = "/json";
        public const string MakesJsonTemplate = "/json/{year}";
        public const string ModelsJsonTemplate = "/json/{year}/{make}";
        public const string EnginesJsonTemplate = "/json/{year}/{make}/{model}";
        public const string EngineJsonTemplate = "/json/{year}/{make}/{model}/{engine}";

        public static readonly UriTemplate Years = new UriTemplate(YearsTemplate);
        public static readonly UriTemplate Makes = new UriTemplate(MakesTemplate);
        public static readonly UriTemplate Models = new UriTemplate(ModelsTemplate);
        public static readonly UriTemplate Engines = new UriTemplate(EnginesTemplate);
        public static readonly UriTemplate Engine = new UriTemplate(EngineTemplate);

        public static readonly UriTemplate YearsJson = new UriTemplate(YearsJsonTemplate);
        public static readonly UriTemplate MakesJson = new UriTemplate(MakesJsonTemplate);
        public static readonly UriTemplate ModelsJson = new UriTemplate(ModelsJsonTemplate);
        public static readonly UriTemplate EnginesJson = new UriTemplate(EnginesJsonTemplate);
        public static readonly UriTemplate EngineJson = new UriTemplate(EngineJsonTemplate);

        public static IEnumerable<KeyValuePair<UriTemplate, object>> GetTemplates()
        {
            var fields = typeof(AutomobileServiceConstants).GetFields(BindingFlags.Public | BindingFlags.Static);
            foreach (var fi in fields)
            {
                // if its a const string add it.
                if (fi.IsLiteral && !fi.IsInitOnly && fi.FieldType == typeof(string))
                {
                    yield return new KeyValuePair<UriTemplate, object>(
                        new UriTemplate(fi.GetValue(null) as string), fi.Name);
                }
            }
        }
    }
}