using System.IO;
using System.Net;
using System.Text;
using System.Web.Mvc;
using System.Web.Script.Serialization;
using System.Xml.Serialization;
using MvcAutomobileService.Helpers;

namespace MvcAutomobileService.Controllers
{
    public class RestController : Controller
    {
        private string _baseUri;
        public string BaseUri
        {
            get
            {
                if (_baseUri == null)
                {
                    var baseUri = new StringBuilder();
                    if (Request != null)
                    {
                        baseUri.Append(Request.Url.Scheme);
                        baseUri.Append("://");
                        baseUri.Append(Request.Url.Host);
                        if (Request.Url.Port > 0)
                        {
                            baseUri.Append(":");
                            baseUri.Append(Request.Url.Port);
                        }
                        var appPath = Request.ApplicationPath;
                        if (Request.ApplicationPath.EndsWith("/"))
                        {
                            appPath = appPath.Remove(appPath.Length - 1);
                        }
                        baseUri.Append(appPath);
                    }
                    _baseUri = baseUri.ToString();
                }
                return _baseUri;
            }
        }

        public ActionResult JsonAllowGet(object data)
        {
            return Json(data, JsonRequestBehavior.AllowGet);
        }

        public T GetPostedJson<T>()
        {
            var streamReader = new StreamReader(Request.InputStream);
            var serializer = new JavaScriptSerializer();
            return serializer.Deserialize<T>(streamReader.ReadToEnd());
        }

        public ActionResult Xml(object data)
        {
            return new XmlResult(data.GetType(), data);
        }

        public ActionResult Xml<T>(T data)
        {
            return new XmlResult(typeof(T), data);
        }

        public T GetPostedXml<T>()
        {
            var serializer = new XmlSerializer(typeof (T));
            return (T)serializer.Deserialize(Request.InputStream);
        }

        public ActionResult HtmlView(string filename)
        {
            return File("~/Views/" + filename, "text/html");
        }

        public ActionResult HttpStatusView(HttpStatusCode code)
        {
            Response.StatusCode = (int)code;
            return HtmlView((int)code + ".html");
        }
        
    }
}