using System;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Mvc;
using System.Xml.Serialization;

namespace MvcAutomobileService.Helpers
{
    public class XmlResult : ActionResult
    {
        public XmlResult()
        {
        }

        public XmlResult(Type dataType, object data)
        {
            DataType = dataType;
            Data = data;
        }

        public string ContentType { get; set; }
        public Encoding ContentEncoding { get; set; }
        public object Data { get; set; }
        public Type DataType { get; set; }

        public override void ExecuteResult(ControllerContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            HttpResponseBase response = context.HttpContext.Response;
            if (!string.IsNullOrEmpty(this.ContentType))
            {
                response.ContentType = this.ContentType;
            }
            else
            {
                response.ContentType = "text/xml";
            }
            if (this.ContentEncoding != null)
            {
                response.ContentEncoding = this.ContentEncoding;
            }
            if (this.Data != null && this.DataType != null)
            {
                var serializer = new XmlSerializer(DataType);
                var sw = new Utf8StringWriter();
                serializer.Serialize(sw, this.Data);
                response.Write(sw.GetStringBuilder().ToString());
            }
        }
    }

    public sealed class Utf8StringWriter : StringWriter
    {
        public override Encoding Encoding { get { return Encoding.UTF8; } }
    }
}