﻿using System;

namespace WcfRestUtilities.Logger
{
    /// <summary>
    /// A implementation of ILogger that logs all data to the console output stream.
    /// </summary>
    public class ConsoleLogger : ILogger
    {
        /// <summary>
        /// Writes the message to the console output stream.
        /// </summary>
        /// <param name="message">The message to be written</param>
        public void Write(string message)
        {
            Console.Write(message);
        }

        /// <summary>
        /// Writes a line of data to the console output stream.
        /// </summary>
        /// <param name="message">The message to be written to its own line.</param>
        public void WriteLine(string message)
        {
            Console.WriteLine(message);
        }

        /// <summary>
        /// Writes an exception to the console output stream in Red text.
        /// </summary>
        /// <param name="exception">The exception to be written.</param>
        public void WriteError(Exception exception)
        {
            WriteError(null, exception);
        }

        /// <summary>
        /// Writes a message and exception details to the console output stream in Red text.
        /// </summary>
        /// <param name="message">The additional message to be added to the output</param>
        /// <param name="exception">The exception to be written.</param>
        public void WriteError(string message, Exception exception)
        {
            var origColor = Console.ForegroundColor;
            Console.ForegroundColor = ConsoleColor.Red;

            if (!string.IsNullOrEmpty(message))
            {
                Console.WriteLine(message);
                Console.WriteLine();
            }

            Console.WriteLine(exception.ToString());
            Console.ForegroundColor = origColor;
        }
    }
}