using System;

namespace WcfRestUtilities.Logger
{
    /// <summary>
    /// A basic logging interface used by the SafeWebServiceHost
    /// </summary>
    public interface ILogger
    {
        /// <summary>
        /// Writes a simple message to the log.
        /// </summary>
        /// <param name="message">The data to write.</param>
        void Write(string message);
        /// <summary>
        /// Writes a message on its own line to the log.
        /// </summary>
        /// <param name="message">The data to write.</param>
        void WriteLine(string message);

        /// <summary>
        /// Writes an exception to the log.
        /// </summary>
        /// <param name="exception">The exception to be written</param>
        void WriteError(Exception exception);
        /// <summary>
        /// Writes an exception with additional details to the log.
        /// </summary>
        /// <param name="message">The additional details to be appended to the exception</param>
        /// <param name="exception">The exception to be written</param>
        void WriteError(string message, Exception exception);
    }
}