﻿using System.Collections.Generic;
using System.ComponentModel;

namespace WcfRestUtilities
{
    /// <summary>
    /// This class is a simple derived version of Dictionary&lt;string, string&gt; that contains a new AddValues property that
    /// generates key/values for an object's properties.
    /// </summary>
    public class TemplateValueDictionary : Dictionary<string, string>
    {
        /// <summary>
        /// Initializes a new instance of TemplateValueDictionary
        /// </summary>
        public TemplateValueDictionary()
        {
        }

        /// <summary>
        /// Initializes a new instance of TemplateValueDictionary with the default contents of dictionary
        /// </summary>
        /// <param name="dictionary">The default contents of the new TemplateValueDictionary</param>
        public TemplateValueDictionary(IDictionary<string, string> dictionary)
            : base(dictionary)
        {
        }

        /// <summary>
        /// Creates a new instance of the TemplateValueDictionary and add the properties of values as key/values pairs.
        /// </summary>
        /// <param name="values">The object to be reflected over to find properties</param>
        public TemplateValueDictionary(object values)
        {
            this.AddValues(values);
        }

        /// <summary>
        /// Uses reflection to get a list of properties and populates the dictionary with key/value pairs for each property.
        /// </summary>
        /// <param name="values">The object to be reflected over to find properties</param>
        public void AddValues(object values)
        {
            if (values != null)
            {
                foreach (PropertyDescriptor descriptor in TypeDescriptor.GetProperties(values))
                {
                    object val = descriptor.GetValue(values);
                    string valString = null;
                    if (val != null)
                    {
                        valString = val.ToString();
                    }
                    this.Add(descriptor.Name, valString);
                }
            }
        }
    }
}