using System;
using System.Runtime.Serialization;
using System.Xml;
using System.Xml.Serialization;

namespace WcfRestUtilities
{
    /// <summary>
    /// A very basic implementation of <see cref="System.Runtime.Serialization.XmlObjectSerializer" /> that composes an <see cref="System.Xml.Serialization.XmlSerializer" />.  This class
    /// is used as a parameter in the <see cref="System.ServiceModel.Channels.Message" />.CreateMessage function.
    /// </summary>
    public class XmlSerializerObjectSerializer : XmlObjectSerializer
    {
        /// <summary>
        /// The inner XmlSerializer to be composed.
        /// </summary>
        private readonly XmlSerializer _serializer;
        /// <summary>
        /// Initializes a new instance of the XmlSerializerObjectSerializer with the specified objectType.
        /// </summary>
        /// <param name="objectType">The type to be used when creating the inner <see cref="System.Xml.Serialization.XmlSerializer" />.</param>
        public XmlSerializerObjectSerializer(Type objectType)
        {
            _serializer = new XmlSerializer(objectType);
        }

        /// <summary>
        /// Does nothing.
        /// </summary>
        public override void WriteStartObject(XmlDictionaryWriter writer, object graph)
        {
        }

        /// <summary>
        /// Writes the object graph to the writer using the inner <see cref="System.Xml.Serialization.XmlSerializer" />.
        /// </summary>
        /// <param name="writer">The writer to write the graph to.</param>
        /// <param name="graph">The object to be serialized.</param>
        public override void WriteObjectContent(XmlDictionaryWriter writer, object graph)
        {
            this._serializer.Serialize((XmlWriter) writer, graph);
        }

        /// <summary>
        /// Does nothing.
        /// </summary>
        public override void WriteEndObject(XmlDictionaryWriter writer)
        {
        }

        /// <summary>
        /// Reads an object using the inner <see cref="System.Xml.Serialization.XmlSerializer" />.
        /// </summary>
        /// <param name="reader">The reader that will be used to load the data.</param>
        /// <param name="verifyObjectName">Not used.  This implementation doesn't verify the object.</param>
        /// <returns>The deserialized object</returns>
        public override object ReadObject(XmlDictionaryReader reader, bool verifyObjectName)
        {
            return _serializer.Deserialize(reader);
        }

        /// <summary>
        /// Checks to see if the reader is at the StartElement.
        /// </summary>
        /// <param name="reader">The reader to be tested.</param>
        /// <returns>True if the reader is the start element, otherwise false.</returns>
        public override bool IsStartObject(XmlDictionaryReader reader)
        {
            reader.MoveToElement();
            return reader.IsStartElement();
        }
    }
}