﻿using System;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Windows;
using System.Xml.Serialization;
using AutomobileSampleDataModel;
using Microsoft.Http;

namespace WpfAutomobileClient
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private const string baseUri = "http://localhost:9999";
        private const string uriFormat = "http://localhost:9999/{0}";

        public Window1()
        {
            InitializeComponent();
        }

        #region Consume REST using the original WebRequest object
        private void Search_Click(object sender, RoutedEventArgs e)
        {
            string uri = string.Format(uriFormat, this.Year.Text);

            var webRequest = WebRequest.Create(uri);

            var response = webRequest.GetResponse();
            using (var outStream = response.GetResponseStream())
            {
                var xmlSerializer = new XmlSerializer(typeof(Makes));
                this.Results.ItemsSource = (Makes)xmlSerializer.Deserialize(outStream);
            }
        } 
        #endregion

        #region Comsume REST with WCF Mocking
        [ServiceContract]
        public interface IAutomobileService
        {
            [OperationContract]
            [WebGet(UriTemplate = "/{year}")]
            [XmlSerializerFormat]
            Makes GetMakes(string year);
        }

        private void SearchWcf_Click(object sender, RoutedEventArgs e)
        {
            var factory = new WebChannelFactory<IAutomobileService>(new Uri(baseUri));
            var proxy = factory.CreateChannel();

            this.Results.ItemsSource = proxy.GetMakes(this.Year.Text);
        } 
        #endregion

        #region Consume REST with the HttpClient from the WCF REST Starter Kit Preview 2
        private void SearchHttp_Click(object sender, RoutedEventArgs e)
        {
            using (var client = new HttpClient())
            {
                string uri = string.Format(uriFormat, this.Year.Text);

                this.Results.ItemsSource = client.Get(uri).EnsureStatusIsSuccessful()
                    .Content.ReadAsXmlSerializable<Makes>();
            }
        }
        #endregion

        #region Clear
        private void Clear_Click(object sender, RoutedEventArgs e)
        {
            this.Results.ItemsSource = null;
        } 
        #endregion
    }
}
